import torch
import torch.nn as nn
from typing import List, Union, Optional
from transformers import AutoModelForSequenceClassification, AutoTokenizer


class Model:
    def __init__(
        self,
        model_path: str,
        **kwargs
    ):
        self.model = AutoModelForSequenceClassification.from_pretrained(model_path, **kwargs)
        self.tokenizer = AutoTokenizer.from_pretrained(model_path)
        self.config = self.model.config
    
    def get_id2label(
        self
    ):
        id2label = self.config.id2label
        return id2label

    def get_label2id(
        self
    ):
        label2id = self.config.label2id
        return label2id

    def predict(
        self,
        text: Union[str, List[str]],
        batch_size: Optional[int] = 1
    ):
        max_length = self.config.max_position_embeddings
        device = self.model.device
        id2label = self.get_id2label()

        if isinstance(text, str):
            text = [text]

        res = []

        for i in range(0, len(text), batch_size):
            batch_text = text[i:i + batch_size]
            inputs = self.tokenizer(
                batch_text, 
                return_tensors="pt", 
                padding='longest', 
                truncation=True, 
                max_length=max_length
            )

            with torch.no_grad():
                logits = self.model(**inputs.to(device))["logits"]

            scores = nn.functional.softmax(logits, dim=-1)
            pred_scores, pred_indices = torch.max(scores, dim=-1)
            pred_scores = pred_scores.cpu().tolist()
            pred_indices = pred_indices.cpu().tolist()

            for score, idx in zip(pred_scores, pred_indices):
                score = round(score, 10)
                label = id2label[idx]
                res_dict = {"label": label, "confidence_score": score}
                res.append(res_dict)

        return res


def load_model(model_path: str = "MusubiAI/ZHLID", **kwargs):
    return Model(model_path=model_path, **kwargs)




import pandas as pd
import json
from tqdm import tqdm
from pathlib import Path
from src.zhlid import load_model


folder = "benchmark/eval_data"
save_path = "benchmark/res/ZHLID_eval_results.jsonl"
model = load_model("MusubiAI/ZHLID", device_map="auto")
files = Path(folder).glob("*.jsonl")

BATCH_SIZE = 5


for file in files:
    print("Start evaluating {} task".format(file.stem))
    df = pd.read_json(file, lines=True)
    num_batches = (len(df) + BATCH_SIZE - 1) // BATCH_SIZE

    start_idx = 0
    correct = 0
    for i in tqdm(range(num_batches)):
        texts = df.iloc[start_idx:start_idx+BATCH_SIZE]["text"].to_list()
        ground_label = df.iloc[start_idx:start_idx+BATCH_SIZE]["ground_label"].to_list()
        res = model.predict(texts, batch_size=BATCH_SIZE)

        for j, item in enumerate(res):
            pred_label = item["label"]
            if pred_label == ground_label[j]:
                correct += 1

        start_idx += BATCH_SIZE

    acc = correct / len(df)
    data = {"lang": file.stem, "accuracy": acc}
    with open(save_path, "a+", encoding="utf-8") as f:
        f.write(json.dumps(data, ensure_ascii=False) + "\n")